"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cancellableWait = exports.raceWithAbort = exports.isCancelError = exports.createCancelError = exports.throwIfAborted = void 0;
class AbortError extends Error {
    constructor(message) {
        super(message ?? 'This operation was aborted');
        this.name = 'AbortError';
    }
}
const throwIfAborted = (signal) => {
    if (signal?.aborted) {
        throw signal.reason ?? (0, exports.createCancelError)();
    }
};
exports.throwIfAborted = throwIfAborted;
const createCancelError = (message) => {
    const controller = new AbortController();
    controller.abort();
    if (message && controller.signal.reason) {
        Object.defineProperty(controller.signal.reason, 'message', {
            get() {
                return message;
            },
            configurable: true,
            enumerable: true,
        });
    }
    return controller.signal.reason ?? new AbortError(message);
};
exports.createCancelError = createCancelError;
function isCancelError(error) {
    return error?.name === 'AbortError';
}
exports.isCancelError = isCancelError;
async function raceWithAbort(promise, signal) {
    if (signal.aborted) {
        return Promise.reject(signal.reason ?? (0, exports.createCancelError)());
    }
    let abortListener;
    const pendingPromise = new Promise((_resolve, reject) => {
        abortListener = () => reject(signal.reason ?? (0, exports.createCancelError)());
        signal.addEventListener('abort', abortListener, { once: true });
    });
    try {
        return await Promise.race([pendingPromise, promise]);
    }
    finally {
        abortListener && signal.removeEventListener('abort', abortListener);
    }
}
exports.raceWithAbort = raceWithAbort;
async function cancellableWait(ms, signal) {
    await raceWithAbort(new Promise((resolve) => {
        setTimeout(resolve, ms);
    }), signal);
}
exports.cancellableWait = cancellableWait;
//# sourceMappingURL=cancellable-promise.js.map